"""Tests for Trained Model."""

from unittest.mock import patch

import numpy as np
from sotai import (
    DatasetSplit,
    FeatureType,
    LinearConfig,
    LossType,
    Metric,
    NumericalFeatureConfig,
    PipelineConfig,
    TargetType,
    TrainedModel,
    TrainedModelMetadata,
    TrainingConfig,
    TrainingResults,
)
from sotai.models import CalibratedLinear

from .utils import construct_trained_model


def test_trained_classification_model_predict(fixture_data, fixture_feature_configs):
    """Tests the predict function on a trained model."""
    trained_model = construct_trained_model(
        TargetType.CLASSIFICATION, fixture_data, fixture_feature_configs
    )
    predictions, probabilities = trained_model.predict(fixture_data)
    assert isinstance(predictions, np.ndarray)
    assert len(predictions) == len(fixture_data)
    assert isinstance(probabilities, np.ndarray)
    assert len(probabilities) == len(fixture_data)


def test_trained_regression_model_predict(fixture_data, fixture_feature_configs):
    """Tests the predict function on a trained model."""
    trained_model = construct_trained_model(
        TargetType.REGRESSION, fixture_data, fixture_feature_configs
    )
    predictions, _ = trained_model.predict(fixture_data)
    assert isinstance(predictions, np.ndarray)
    assert len(predictions) == len(fixture_data)


def test_trained_model_save_load(
    fixture_data,
    fixture_feature_configs,
    tmp_path,
):
    """Tests that a `TrainedModel` can be successfully saved and then loaded."""
    trained_model = construct_trained_model(
        TargetType.CLASSIFICATION, fixture_data, fixture_feature_configs
    )
    trained_model.save(tmp_path)
    loaded_trained_model = TrainedModel.load(tmp_path)
    assert isinstance(loaded_trained_model, TrainedModel)
    assert loaded_trained_model.dict(
        exclude={"model", "saved_filepath"}
    ) == trained_model.dict(exclude={"model", "saved_filepath"})
    assert isinstance(loaded_trained_model.model, CalibratedLinear)


@patch("sotai.trained_model.download_trained_model")
@patch(
    "sotai.trained_model.get_trained_model_metadata",
    return_value=TrainedModelMetadata(
        **{
            "id": 0,
            "dataset_id": 0,
            "model_config": LinearConfig(),
            "training_config": TrainingConfig(
                loss_type=LossType.MSE,
                epochs=1,
                batch_size=1,
                learning_rate=1,
            ),
            "training_results": TrainingResults(
                training_time=1,
                train_loss_by_epoch=[1],
                train_primary_metric_by_epoch=[1],
                val_loss_by_epoch=[1],
                val_primary_metric_by_epoch=[1],
                evaluation_time=1,
                test_loss=1,
                test_primary_metric=1,
                feature_analyses={},
                linear_coefficients={},
            ),
            "uuid": "test_uuid",
            "pipeline_config": PipelineConfig(
                id=0,
                target="target",
                target_type=TargetType.CLASSIFICATION,
                primary_metric=Metric.MSE,
                feature_configs={
                    "age": NumericalFeatureConfig(
                        name="age",
                        type=FeatureType.NUMERICAL,
                    ),
                },
                shuffle_data=False,
                drop_empty_percentage=70,
                dataset_split=DatasetSplit(train=80, val=10, test=10),
            ),
        }
    ),
)
def test_load_from_hosted(
    mock_get_trained_model,
    mock_download_trained_model,
    tmp_path,
    fixture_data,
    fixture_feature_configs,
):
    """Test loading a trained model from hosted."""
    trained_model = construct_trained_model(
        TargetType.CLASSIFICATION, fixture_data, fixture_feature_configs
    )
    trained_model.save(tmp_path)
    mock_download_trained_model.return_value = f"{tmp_path}/trained_ptcm_model.pt"
    trained_model = TrainedModel.from_hosted("test_uuid")
    mock_get_trained_model.assert_called_once()
    mock_download_trained_model.assert_called_once()
    assert isinstance(trained_model, TrainedModel)
    assert trained_model.training_config.epochs == 1
    assert trained_model.training_config.batch_size == 1
    assert trained_model.training_config.learning_rate == 1
    assert trained_model.training_config.loss_type == LossType.MSE
    assert trained_model.training_results.training_time == 1
    assert trained_model.training_results.train_loss_by_epoch == [1]
    assert trained_model.training_results.train_primary_metric_by_epoch == [1]
    assert trained_model.training_results.val_loss_by_epoch == [1]
    assert trained_model.model_config.output_calibration is False



"""This module contains functions for external models to interact with the SOTAI API."""
import os

import numpy as np
import pandas as pd

from .api import post_external_inference


def shap(
    inference_data: pd.DataFrame,
    shapley_values: np.ndarray,
    base_values: np.ndarray,
    name: str,
) -> str:
    """Uploads the shapley values, base values, and inference data to the SOTAI API.

    Args:
        inference_data: The data used for inference.
        shapley_values: The shapley values for the inference data.
        base_values: The base values for the inference data.
        name: The name of the shapley values.

    Returns:
        The UUID of the uploaded shapley values.
    """

    shapley_values_df = pd.DataFrame(shapley_values)
    base_values_df = pd.DataFrame(base_values)

    external_directory = "/tmp/sotai/external/"
    if not os.path.exists(external_directory):
        os.makedirs(external_directory)
    shapley_value_filepath = "/tmp/sotai/external/shapley_values.csv"
    base_values_filepath = "/tmp/sotai/external/base_values.csv"
    inference_data_filepath = "/tmp/sotai/external/inference_predictions.csv"

    shapley_values_df.to_csv(shapley_value_filepath)
    base_values_df.to_csv(base_values_filepath)
    inference_data.to_csv(inference_data_filepath)

    shap_uuid = post_external_inference(
        external_shapley_value_name=name,
        shap_filepath=shapley_value_filepath,
        base_filepath=base_values_filepath,
        inference_filepath=inference_data_filepath,
    )
    return shap_uuid



"""Tests for api."""
from unittest.mock import patch
import pandas as pd
import numpy as np

from sotai.external import shap


@patch("pandas.DataFrame.to_csv", return_value="test")
@patch("sotai.external.post_external_inference", return_value="test_uuid")
def test_shap(mock_post_external_inference, mock_to_csv):
    """Tests that a pipeline is posted correctly.""" ""

    test_inference_data = pd.DataFrame([[1, 2, 3], [4, 5, 6]], columns=["a", "b", "c"])

    test_shap_values = np.array([[1, 2, 3], [4, 5, 6]])
    test_base_values = np.array([[1, 2, 3], [4, 5, 6]])

    shap(test_inference_data, test_shap_values, test_base_values, "test")

    mock_post_external_inference.assert_called_with(
        external_shapley_value_name="test",
        shap_filepath="/tmp/sotai/external/shapley_values.csv",
        base_filepath="/tmp/sotai/external/base_values.csv",
        inference_filepath="/tmp/sotai/external/inference_predictions.csv",
    )
    mock_to_csv.assert_called()
